@@ -42,12 +42,15 @@ struct _CcGoaPanel
   GoaObject *active_object;
   GoaObject *removed_object;
 
+  GtkWidget *accounts_frame;
   GtkWidget *accounts_listbox;
   GtkWidget *edit_account_dialog;
   GtkWidget *edit_account_headerbar;
+  GtkWidget *more_providers_row;
   GtkWidget *new_account_vbox;
   GtkWidget *notification_label;
   GtkWidget *notification_revealer;
+  GtkWidget *offline_label;
   GtkWidget *providers_listbox;
   GtkWidget *remove_account_button;
   GtkWidget *stack;
@@ -123,6 +126,7 @@ add_provider_row (CcGoaPanel  *self,
                   GoaProvider *provider)
 {
   GIcon *icon;
+  GoaProviderFeatures features;
   GtkWidget *image;
   GtkWidget *label;
   GtkWidget *row;
@@ -131,7 +135,6 @@ add_provider_row (CcGoaPanel  *self,
   gchar *name;
 
   row = gtk_list_box_row_new ();
-  gtk_container_add (GTK_CONTAINER (self->providers_listbox), row);
 
   row_grid = gtk_grid_new ();
   gtk_orientable_set_orientation (GTK_ORIENTABLE (row_grid), GTK_ORIENTATION_HORIZONTAL);
@@ -153,29 +156,91 @@ add_provider_row (CcGoaPanel  *self,
 
   image = gtk_image_new_from_gicon (icon, GTK_ICON_SIZE_DIALOG);
   gtk_container_add (GTK_CONTAINER (row_grid), image);
+  g_object_set (image, "margin", 6, NULL);
 
   markup = g_strdup_printf ("<b>%s</b>", name);
   label = gtk_label_new (NULL);
   gtk_label_set_markup (GTK_LABEL (label), markup);
   gtk_container_add (GTK_CONTAINER (row_grid), label);
 
-  gtk_widget_show_all (row);
+  /* Check if the row should be shown initially */
+  features = goa_provider_get_provider_features (provider);
+
+  if ((features & GOA_PROVIDER_FEATURE_BRANDED) != 0)
+    gtk_widget_show_all (row);
+
+  gtk_container_add (GTK_CONTAINER (self->providers_listbox), row);
 
   g_free (markup);
   g_free (name);
   g_object_unref (icon);
 }
 
+static gint
+sort_providers_func (GtkListBoxRow *a,
+                     GtkListBoxRow *b,
+                     gpointer       user_data)
+{
+  GoaProvider *a_provider, *b_provider;
+  CcGoaPanel *self;
+  gboolean a_branded, b_branded;
+
+  self = user_data;
+
+  if (a == GTK_LIST_BOX_ROW (self->more_providers_row))
+    return 1;
+  else if (b == GTK_LIST_BOX_ROW (self->more_providers_row))
+    return -1;
+
+  a_provider = g_object_get_data (G_OBJECT (a), "goa-provider");
+  b_provider = g_object_get_data (G_OBJECT (b), "goa-provider");
+
+  a_branded = (goa_provider_get_provider_features (a_provider) & GOA_PROVIDER_FEATURE_BRANDED) != 0;
+  b_branded = (goa_provider_get_provider_features (b_provider) & GOA_PROVIDER_FEATURE_BRANDED) != 0;
+
+  if (a_branded != b_branded)
+    {
+      if (a_branded)
+        return -1;
+      else
+        return 1;
+    }
+
+  return gtk_list_box_row_get_index (b) - gtk_list_box_row_get_index (a);
+}
+
 static void
-on_provider_row_activated (CcGoaPanel    *self,
-                           GtkListBoxRow *activated_row)
+show_non_branded_providers (CcGoaPanel *self)
+{
+  GList *children, *l;
+
+  children = gtk_container_get_children (GTK_CONTAINER (self->providers_listbox));
+
+  for (l = children; l != NULL; l = l->next)
+    {
+      GoaProvider *provider = g_object_get_data (l->data, "goa-provider");
+
+      if (!provider)
+        continue;
+
+      if ((goa_provider_get_provider_features (provider) & GOA_PROVIDER_FEATURE_BRANDED) == 0)
+        gtk_widget_show_all (l->data);
+    }
+
+  gtk_widget_hide (self->more_providers_row);
+
+  g_list_free (children);
+}
+
+static void
+add_account (CcGoaPanel  *self,
+             GoaProvider *provider,
+             GVariant    *preseed)
 {
-  GoaProvider *provider;
   GoaObject *object;
   GError *error;
 
   error = NULL;
-  provider = g_object_get_data (G_OBJECT (activated_row), "goa-provider");
 
   gtk_container_foreach (GTK_CONTAINER (self->new_account_vbox),
                          (GtkCallback) gtk_widget_destroy,
@@ -186,6 +251,9 @@ on_provider_row_activated (CcGoaPanel    *self,
   /* Move to the new account page */
   gtk_stack_set_visible_child_name (GTK_STACK (self->stack), "new-account");
 
+  /* Reset the dialog size */
+  gtk_window_resize (GTK_WINDOW (self->edit_account_dialog), 1, 1);
+
   /* This spins gtk_dialog_run() */
   object = goa_provider_add_account (provider,
                                      self->client,
@@ -193,12 +261,33 @@ on_provider_row_activated (CcGoaPanel    *self,
                                      GTK_BOX (self->new_account_vbox),
                                      &error);
 
+  if (preseed)
+    goa_provider_set_preseed_data (provider, preseed);
+
   if (object == NULL)
     gtk_widget_hide (self->edit_account_dialog);
   else
     show_page_account (self, object);
 }
 
+static void
+on_provider_row_activated (CcGoaPanel    *self,
+                           GtkListBoxRow *activated_row)
+{
+  GoaProvider *provider;
+
+  /* Show More row */
+  if (activated_row == GTK_LIST_BOX_ROW (self->more_providers_row))
+    {
+      show_non_branded_providers (self);
+      return;
+    }
+
+  provider = g_object_get_data (G_OBJECT (activated_row), "goa-provider");
+
+  add_account (self, provider, NULL);
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 static gint
@@ -231,20 +320,16 @@ command_add (CcGoaPanel *panel,
 
   switch (g_variant_n_children (parameters))
     {
-      case 4:
-        g_variant_get_child (parameters, 3, "v", &preseed);
       case 3:
-        g_variant_get_child (parameters, 2, "v", &v);
+        g_variant_get_child (parameters, 2, "v", &preseed);
+      case 2:
+        g_variant_get_child (parameters, 1, "v", &v);
         if (g_variant_is_of_type (v, G_VARIANT_TYPE_STRING))
           provider_name = g_variant_get_string (v, NULL);
         else
           g_warning ("Wrong type for the second argument (provider name) GVariant, expected 's' but got '%s'",
                      (gchar *)g_variant_get_type (v));
         g_variant_unref (v);
-      case 2:
-        /* Nothing to see here, move along */
-      case 1:
-        /* No flag to handle here */
         break;
       default:
         g_warning ("Unexpected parameters found, ignore request");
@@ -259,6 +344,8 @@ command_add (CcGoaPanel *panel,
           g_warning ("Unable to get a provider for type '%s'", provider_name);
           goto out;
         }
+
+      add_account (panel, provider, preseed);
     }
 
 out:
@@ -306,6 +393,17 @@ cc_goa_panel_set_property (GObject *object,
   G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 }
 
+static void
+cc_goa_panel_dispose (GObject *object)
+{
+  CcGoaPanel *panel = CC_GOA_PANEL (object);
+
+  /* Must be destroyed in dispose, not finalize. */
+  g_clear_pointer (&panel->edit_account_dialog, gtk_widget_destroy);
+
+  G_OBJECT_CLASS (cc_goa_panel_parent_class)->dispose (object);
+}
+
 static void
 cc_goa_panel_finalize (GObject *object)
 {
@@ -339,9 +437,17 @@ cc_goa_panel_init (CcGoaPanel *panel)
                                 cc_list_box_update_header_func,
                                 NULL,
                                 NULL);
+  gtk_list_box_set_sort_func (GTK_LIST_BOX (panel->providers_listbox),
+                              sort_providers_func,
+                              panel,
+                              NULL);
 
   monitor = g_network_monitor_get_default();
 
+  g_object_bind_property (monitor, "network-available",
+                          panel->offline_label, "visible",
+                          G_BINDING_SYNC_CREATE | G_BINDING_INVERT_BOOLEAN);
+
   g_object_bind_property (monitor, "network-available",
                           panel->providers_listbox, "sensitive",
                           G_BINDING_SYNC_CREATE);
@@ -376,7 +482,7 @@ cc_goa_panel_init (CcGoaPanel *panel)
   fill_accounts_listbox (panel);
   goa_provider_get_all (get_all_providers_cb, panel);
 
-  gtk_widget_show_all (GTK_WIDGET (panel));
+  gtk_widget_show (GTK_WIDGET (panel));
 }
 
 static const char *
@@ -411,18 +517,22 @@ cc_goa_panel_class_init (CcGoaPanelClass *klass)
   object_class->set_property = cc_goa_panel_set_property;
   object_class->finalize = cc_goa_panel_finalize;
   object_class->constructed = cc_goa_panel_constructed;
+  object_class->dispose = cc_goa_panel_dispose;
 
   g_object_class_override_property (object_class, PROP_PARAMETERS, "parameters");
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/control-center/online-accounts/online-accounts.ui");
 
+  gtk_widget_class_bind_template_child (widget_class, CcGoaPanel, accounts_frame);
   gtk_widget_class_bind_template_child (widget_class, CcGoaPanel, accounts_listbox);
   gtk_widget_class_bind_template_child (widget_class, CcGoaPanel, accounts_vbox);
   gtk_widget_class_bind_template_child (widget_class, CcGoaPanel, edit_account_dialog);
   gtk_widget_class_bind_template_child (widget_class, CcGoaPanel, edit_account_headerbar);
+  gtk_widget_class_bind_template_child (widget_class, CcGoaPanel, more_providers_row);
   gtk_widget_class_bind_template_child (widget_class, CcGoaPanel, new_account_vbox);
   gtk_widget_class_bind_template_child (widget_class, CcGoaPanel, notification_label);
   gtk_widget_class_bind_template_child (widget_class, CcGoaPanel, notification_revealer);
+  gtk_widget_class_bind_template_child (widget_class, CcGoaPanel, offline_label);
   gtk_widget_class_bind_template_child (widget_class, CcGoaPanel, providers_listbox);
   gtk_widget_class_bind_template_child (widget_class, CcGoaPanel, remove_account_button);
   gtk_widget_class_bind_template_child (widget_class, CcGoaPanel, stack);
@@ -494,6 +604,9 @@ show_page_account (CcGoaPanel  *panel,
   gtk_header_bar_set_title (GTK_HEADER_BAR (panel->edit_account_headerbar), title);
   g_free (title);
 
+  /* Reset the dialog size */
+  gtk_window_resize (GTK_WINDOW (panel->edit_account_dialog), 1, 1);
+
   gtk_widget_show_all (panel->accounts_vbox);
   gtk_widget_show (panel->edit_account_dialog);
 
@@ -566,6 +679,61 @@ fill_accounts_listbox (CcGoaPanel *self)
   g_list_free_full (accounts, g_object_unref);
 }
 
+/* ---------------------------------------------------------------------------------------------------- */
+
+typedef void (*RowForAccountCallback) (CcGoaPanel *self, GtkWidget *row, GList *other_rows);
+
+static void
+hide_row_for_account (CcGoaPanel *self, GtkWidget *row, GList *other_rows)
+{
+  gtk_widget_hide (row);
+  gtk_widget_set_visible (self->accounts_frame, other_rows != NULL);
+}
+
+static void
+remove_row_for_account (CcGoaPanel *self, GtkWidget *row, GList *other_rows)
+{
+  gtk_widget_destroy (row);
+  gtk_widget_set_visible (self->accounts_frame, other_rows != NULL);
+}
+
+static void
+show_row_for_account (CcGoaPanel *self, GtkWidget *row, GList *other_rows)
+{
+  gtk_widget_show (row);
+  gtk_widget_show (self->accounts_frame);
+}
+
+static void
+modify_row_for_account (CcGoaPanel *self,
+                        GoaObject *object,
+                        RowForAccountCallback callback)
+{
+  GList *children, *l;
+
+  children = gtk_container_get_children (GTK_CONTAINER (self->accounts_listbox));
+
+  for (l = children; l != NULL; l = l->next)
+    {
+      GoaObject *row_object;
+
+      row_object = g_object_get_data (G_OBJECT (l->data), "goa-object");
+      if (row_object == object)
+        {
+          GtkWidget *row = GTK_WIDGET (l->data);
+
+          children = g_list_remove_link (children, l);
+          callback (self, row, children);
+          g_list_free (l);
+          break;
+        }
+    }
+
+  g_list_free (children);
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 static void
 on_account_added (GoaClient *client,
                   GoaObject *object,
@@ -603,6 +771,8 @@ on_account_added (GoaClient *client,
       gtk_image_set_from_gicon (GTK_IMAGE (icon), gicon, GTK_ICON_SIZE_DIALOG);
     }
 
+  g_object_set (icon, "margin", 6, NULL);
+
   gtk_container_add (GTK_CONTAINER (box), icon);
 
   /* The name of the provider */
@@ -622,6 +792,7 @@ on_account_added (GoaClient *client,
   /* "Needs attention" icon */
   icon = gtk_image_new_from_icon_name ("dialog-warning-symbolic", GTK_ICON_SIZE_BUTTON);
   gtk_widget_set_no_show_all (icon, TRUE);
+  g_object_set (icon, "margin_end", 30, NULL);
   g_object_bind_property (goa_object_peek_account (object),
                           "attention-needed",
                           icon,
@@ -637,6 +808,7 @@ on_account_added (GoaClient *client,
   /* Add to the listbox */
   gtk_container_add (GTK_CONTAINER (self->accounts_listbox), row);
   gtk_widget_show_all (row);
+  gtk_widget_show (self->accounts_frame);
 
   g_clear_pointer (&title, g_free);
   g_clear_object (&gicon);
@@ -661,24 +833,7 @@ on_account_removed (GoaClient *client,
                     gpointer   user_data)
 {
   CcGoaPanel *self = user_data;
-  GList *children, *l;
-
-  children = gtk_container_get_children (GTK_CONTAINER (self->accounts_listbox));
-
-  for (l = children; l != NULL; l = l->next)
-    {
-      GoaObject *row_object;
-
-      row_object = GOA_OBJECT (g_object_get_data (l->data, "goa-object"));
-
-      if (row_object == object)
-        {
-          gtk_widget_destroy (l->data);
-          break;
-        }
-    }
-
-  g_list_free (children);
+  modify_row_for_account (self, object, remove_row_for_account);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -710,33 +865,6 @@ get_all_providers_cb (GObject      *source,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-
-static GtkWidget *
-get_row_for_account (CcGoaPanel *self,
-                     GoaObject *object)
-{
-  GtkWidget *row;
-  GList *children, *l;
-
-  row = NULL;
-  children = gtk_container_get_children (GTK_CONTAINER (self->accounts_listbox));
-
-  for (l = children; l != NULL; l = l->next)
-    {
-      GoaObject *row_object;
-
-      row_object = g_object_get_data (G_OBJECT (l->data), "goa-object");
-      if (row_object == object)
-        {
-          row = l->data;
-          break;
-        }
-    }
-
-  g_list_free (children);
-  return row;
-}
-
 static void
 cancel_notification_timeout (CcGoaPanel *self)
 {
@@ -795,12 +923,8 @@ static void
 on_undo_button_clicked (GtkButton  *button,
                         CcGoaPanel *self)
 {
-  GtkWidget *row;
-
   /* Simply show the account row and hide the notification */
-  row = get_row_for_account (self, self->removed_object);
-  gtk_widget_show (row);
-
+  modify_row_for_account (self, self->removed_object, show_row_for_account);
   gtk_revealer_set_reveal_child (GTK_REVEALER (self->notification_revealer), FALSE);
 
   cancel_notification_timeout (self);
@@ -818,7 +942,6 @@ static void
 on_remove_button_clicked (CcGoaPanel *panel)
 {
   GoaAccount *account;
-  GtkWidget *row;
   gchar *label;
 
   if (panel->active_object == NULL)
@@ -838,10 +961,8 @@ on_remove_button_clicked (CcGoaPanel *panel)
   gtk_label_set_markup (GTK_LABEL (panel->notification_label), label);
   gtk_revealer_set_reveal_child (GTK_REVEALER (panel->notification_revealer), TRUE);
 
-  row = get_row_for_account (panel, panel->removed_object);
-
+  modify_row_for_account (panel, panel->removed_object, hide_row_for_account);
   gtk_widget_hide (panel->edit_account_dialog);
-  gtk_widget_hide (row);
 
   panel->remove_account_timeout_id = g_timeout_add_seconds (10, on_remove_account_timeout, panel);
 
