@@ -124,7 +124,7 @@ file_chooser_response (GtkDialog     *chooser,
 {
         gchar *filename;
         GError *error;
-        GdkPixbuf *pixbuf;
+        GdkPixbuf *pixbuf, *pixbuf2;
 
         if (response != GTK_RESPONSE_ACCEPT) {
                 gtk_widget_destroy (GTK_WIDGET (chooser));
@@ -141,10 +141,13 @@ file_chooser_response (GtkDialog     *chooser,
         }
         g_free (filename);
 
+        pixbuf2 = gdk_pixbuf_apply_embedded_orientation (pixbuf);
+        g_object_unref (pixbuf);
+
         gtk_widget_destroy (GTK_WIDGET (chooser));
 
-        um_photo_dialog_crop (um, pixbuf);
-        g_object_unref (pixbuf);
+        um_photo_dialog_crop (um, pixbuf2);
+        g_object_unref (pixbuf2);
 }
 
 static void
@@ -495,13 +498,21 @@ skip_faces:
 static void
 popup_icon_menu (GtkToggleButton *button, UmPhotoDialog *um)
 {
+        GtkWindow *window;
+
+        window = GTK_WINDOW (gtk_widget_get_toplevel (um->photo_popup));
         if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)) && !gtk_widget_get_visible (um->photo_popup)) {
-                gtk_menu_popup (GTK_MENU (um->photo_popup),
-                                NULL, NULL,
-                                (GtkMenuPositionFunc) popup_menu_below_button, um->popup_button,
-                                0, gtk_get_current_event_time ());
+                gtk_menu_popup_at_widget (GTK_MENU (um->photo_popup),
+                                          GTK_WIDGET (button),
+                                          GDK_GRAVITY_NORTH_WEST,
+                                          GDK_GRAVITY_SOUTH_WEST,
+                                          NULL);
+
+                gtk_window_set_transient_for (window, GTK_WINDOW (gtk_widget_get_toplevel (um->popup_button)));
         } else {
                 gtk_menu_popdown (GTK_MENU (um->photo_popup));
+
+                gtk_window_set_transient_for (window, NULL);
         }
 }
 
